package sun.security.krb5;

import java.util.HashSet;
import java.util.ArrayList;
import java.util.StringTokenizer;
import java.util.List;
import java.util.Set;
import java.net.SocketTimeoutException;
import sun.security.krb5.internal.NetClient;
import java.security.PrivilegedExceptionAction;
import sun.security.krb5.internal.Krb5;
import java.security.PrivilegedActionException;
import sun.security.krb5.internal.KRBError;
import java.util.Iterator;
import java.io.IOException;
import java.util.Locale;
import java.security.AccessController;
import java.security.Security;
import java.security.PrivilegedAction;

public final class KdcComm
{
    private static int defaultKdcRetryLimit;
    private static int defaultKdcTimeout;
    private static int defaultUdpPrefLimit;
    private static final boolean DEBUG;
    private static final String BAD_POLICY_KEY = "krb5.kdc.bad.policy";
    private static int tryLessMaxRetries;
    private static int tryLessTimeout;
    private static KdcComm.BpType badPolicy;
    private String realm;
    
    public static void initStatic() {
        String s = (String)AccessController.doPrivileged(new PrivilegedAction<String>() {
            @Override
            public String run() {
                return Security.getProperty("krb5.kdc.bad.policy");
            }
        });
        if (s != null) {
            String lowerCase = s.toLowerCase(Locale.ENGLISH);
            String[] split = lowerCase.split(":");
            if ("tryless".equals(split[0])) {
                if (split.length > 1) {
                    String[] split2 = split[1].split(",");
                    try {
                        int int1 = Integer.parseInt(split2[0]);
                        if (split2.length > 1) {
                            KdcComm.tryLessTimeout = Integer.parseInt(split2[1]);
                        }
                        KdcComm.tryLessMaxRetries = int1;
                    }
                    catch (NumberFormatException ex) {
                        if (KdcComm.DEBUG) {
                            System.out.println("Invalid krb5.kdc.bad.policy parameter for tryLess: " + lowerCase + ", use default");
                        }
                    }
                }
                KdcComm.badPolicy = KdcComm.BpType.TRY_LESS;
            }
            else if ("trylast".equals(split[0])) {
                KdcComm.badPolicy = KdcComm.BpType.TRY_LAST;
            }
            else {
                KdcComm.badPolicy = KdcComm.BpType.NONE;
            }
        }
        else {
            KdcComm.badPolicy = KdcComm.BpType.NONE;
        }
        int timeString = -1;
        int positiveIntString = -1;
        int positiveIntString2 = -1;
        try {
            Config instance = Config.getInstance();
            timeString = parseTimeString(instance.get(new String[] { "libdefaults", "kdc_timeout" }));
            positiveIntString = parsePositiveIntString(instance.get(new String[] { "libdefaults", "max_retries" }));
            positiveIntString2 = parsePositiveIntString(instance.get(new String[] { "libdefaults", "udp_preference_limit" }));
        }
        catch (Exception ex2) {
            if (KdcComm.DEBUG) {
                System.out.println("Exception in getting KDC communication settings, using default value " + ex2.getMessage());
            }
        }
        KdcComm.defaultKdcTimeout = ((timeString > 0) ? timeString : 30000);
        KdcComm.defaultKdcRetryLimit = ((positiveIntString > 0) ? positiveIntString : 3);
        if (positiveIntString2 < 0) {
            KdcComm.defaultUdpPrefLimit = 1465;
        }
        else if (positiveIntString2 > 32700) {
            KdcComm.defaultUdpPrefLimit = 32700;
        }
        else {
            KdcComm.defaultUdpPrefLimit = positiveIntString2;
        }
        KdcComm.KdcAccessibility.access$000();
    }
    
    public KdcComm(String defaultRealm) throws KrbException {
        if (defaultRealm == null) {
            defaultRealm = Config.getInstance().getDefaultRealm();
            if (defaultRealm == null) {
                throw new KrbException(60, "Cannot find default realm");
            }
        }
        this.realm = defaultRealm;
    }
    
    public byte[] send(byte[] array) throws IOException, KrbException {
        int realmSpecificValue = this.getRealmSpecificValue(this.realm, "udp_preference_limit", KdcComm.defaultUdpPrefLimit);
        return this.send(array, realmSpecificValue > 0 && array != null && array.length > realmSpecificValue);
    }
    
    private byte[] send(byte[] array, boolean b) throws IOException, KrbException {
        if (array == null) {
            return null;
        }
        Config instance = Config.getInstance();
        if (this.realm == null) {
            this.realm = instance.getDefaultRealm();
            if (this.realm == null) {
                throw new KrbException(60, "Cannot find default realm");
            }
        }
        String kdcList = instance.getKDCList(this.realm);
        if (kdcList == null) {
            throw new KrbException("Cannot get kdc for realm " + this.realm);
        }
        Iterator<Object> iterator = KdcComm.KdcAccessibility.access$100(kdcList).iterator();
        if (!iterator.hasNext()) {
            throw new KrbException("Cannot get kdc for realm " + this.realm);
        }
        byte[] array2 = null;
        try {
            array2 = this.sendIfPossible(array, (String)iterator.next(), b);
        }
        catch (Exception ex) {
            boolean n = false;
            while (iterator.hasNext()) {
                try {
                    array2 = this.sendIfPossible(array, (String)iterator.next(), b);
                    n = true;
                    break;
                }
                catch (Exception ex2) {}
            }
            if (n == 0) {
                throw ex;
            }
        }
        if (array2 == null) {
            throw new IOException("Cannot get a KDC reply");
        }
        return array2;
    }
    
    private byte[] sendIfPossible(byte[] array, String str, boolean b) throws IOException, KrbException {
        try {
            byte[] data = this.send(array, str, b);
            KRBError krbError = null;
            try {
                krbError = new KRBError(data);
            }
            catch (Exception ex) {}
            if (krbError != null && krbError.getErrorCode() == 52) {
                data = this.send(array, str, true);
            }
            KdcComm.KdcAccessibility.access$200(str);
            return data;
        }
        catch (Exception ex2) {
            if (KdcComm.DEBUG) {
                System.out.println(">>> KrbKdcReq send: error trying " + str);
                ex2.printStackTrace(System.out);
            }
            KdcComm.KdcAccessibility.access$300(str);
            throw ex2;
        }
    }
    
    private byte[] send(byte[] array, String s, boolean b) throws IOException, KrbException {
        if (array == null) {
            return null;
        }
        int i = 88;
        int j = this.getRealmSpecificValue(this.realm, "max_retries", KdcComm.defaultKdcRetryLimit);
        int k = this.getRealmSpecificValue(this.realm, "kdc_timeout", KdcComm.defaultKdcTimeout);
        if (KdcComm.badPolicy == KdcComm.BpType.TRY_LESS && KdcComm.KdcAccessibility.access$400(s)) {
            if (j > KdcComm.tryLessMaxRetries) {
                j = KdcComm.tryLessMaxRetries;
            }
            if (k > KdcComm.tryLessTimeout) {
                k = KdcComm.tryLessTimeout;
            }
        }
        String s2 = null;
        String str;
        if (s.charAt(0) == '[') {
            int index2 = s.indexOf(93, 1);
            if (index2 == -1) {
                throw new IOException("Illegal KDC: " + s);
            }
            str = s.substring(1, index2);
            if (index2 != s.length() - 1) {
                if (s.charAt(index2 + 1) != ':') {
                    throw new IOException("Illegal KDC: " + s);
                }
                s2 = s.substring(index2 + 2);
            }
        }
        else {
            int index = s.indexOf(58);
            if (index == -1) {
                str = s;
            }
            else if (s.indexOf(58, index + 1) > 0) {
                str = s;
            }
            else {
                str = s.substring(0, index);
                s2 = s.substring(index + 1);
            }
        }
        if (s2 != null) {
            int positiveIntString = parsePositiveIntString(s2);
            if (positiveIntString > 0) {
                i = positiveIntString;
            }
        }
        if (KdcComm.DEBUG) {
            System.out.println(">>> KrbKdcReq send: kdc=" + str + (b ? " TCP:" : " UDP:") + i + ", timeout=" + k + ", number of retries =" + j + ", #bytes=" + array.length);
        }
        KdcComm.KdcCommunication action = new KdcComm.KdcCommunication(str, i, b, k, j, array);
        try {
            byte[] array2 = (byte[])AccessController.doPrivileged(action);
            if (KdcComm.DEBUG) {
                System.out.println(">>> KrbKdcReq send: #bytes read=" + ((array2 != null) ? array2.length : 0));
                return array2;
            }
            return array2;
        }
        catch (PrivilegedActionException ex) {
            Exception exception = ex.getException();
            if ((exception instanceof IOException)) {
                throw (IOException)exception;
            }
            throw (KrbException)exception;
        }
    }
    
    private static int parseTimeString(String s) {
        if (s == null) {
            return -1;
        }
        if (s.endsWith("s")) {
            int positiveIntString = parsePositiveIntString(s.substring(0, s.length() - 1));
            return (positiveIntString < 0) ? -1 : (positiveIntString * 1000);
        }
        return parsePositiveIntString(s);
    }
    
    private int getRealmSpecificValue(String s, String s2, int n) {
        if (s == null) {
            return n;
        }
        int n2 = -1;
        try {
            String value = Config.getInstance().get(new String[] { "realms", s, s2 });
            if (s2.equals("kdc_timeout")) {
                n2 = parseTimeString(value);
            }
            else {
                n2 = parsePositiveIntString(value);
            }
        }
        catch (Exception ex) {}
        if (n2 > 0) {
            n = n2;
        }
        return n;
    }
    
    private static int parsePositiveIntString(String s) {
        if (s == null) {
            return -1;
        }
        int int1;
        try {
            int1 = Integer.parseInt(s);
        }
        catch (Exception ex) {
            return -1;
        }
        if (int1 >= 0) {
            return int1;
        }
        return -1;
    }
    
    static {
        DEBUG = Krb5.DEBUG;
        KdcComm.tryLessMaxRetries = 1;
        KdcComm.tryLessTimeout = 5000;
        initStatic();
    }
    
    private enum BpType
    {
        NONE, 
        TRY_LAST, 
        TRY_LESS;
    }
    
    private static class KdcCommunication implements PrivilegedExceptionAction<byte[]>
    {
        private String kdc;
        private int port;
        private boolean useTCP;
        private int timeout;
        private int retries;
        private byte[] obuf;
        
        public KdcCommunication(String kdc, int port, boolean useTCP, int timeout, int retries, byte[] obuf) {
            this.kdc = kdc;
            this.port = port;
            this.useTCP = useTCP;
            this.timeout = timeout;
            this.retries = retries;
            this.obuf = obuf;
        }
        
        @Override
        public byte[] run() throws IOException, KrbException {
            byte[] receive = null;
            int i = 1;
            while (i <= this.retries) {
                String s = this.useTCP ? "TCP" : "UDP";
                if (KdcComm.DEBUG) {
                    System.out.println(">>> KDCCommunication: kdc=" + this.kdc + " " + s + ":" + this.port + ", timeout=" + this.timeout + ",Attempt =" + i + ", #bytes=" + this.obuf.length);
                }
                try (final NetClient instance = NetClient.getInstance(s, this.kdc, this.port, this.timeout)) {
                    instance.send(this.obuf);
                    receive = instance.receive();
                }
                catch (SocketTimeoutException ex) {
                    if (KdcComm.DEBUG) {
                        System.out.println("SocketTimeOutException with attempt: " + i);
                    }
                    if (i == this.retries) {
                        throw ex;
                    }
                    ++i;
                    continue;
                }
                break;
            }
            return receive;
        }
    }
    
    static class KdcAccessibility
    {
        private static Set<String> bads;
        
        private static synchronized void addBad(String str) {
            if (KdcComm.DEBUG) {
                System.out.println(">>> KdcAccessibility: add " + str);
            }
            KdcComm.KdcAccessibility.bads.add(str);
        }
        
        private static synchronized void removeBad(String str) {
            if (KdcComm.DEBUG) {
                System.out.println(">>> KdcAccessibility: remove " + str);
            }
            KdcComm.KdcAccessibility.bads.remove(str);
        }
        
        private static synchronized boolean isBad(String s) {
            return KdcComm.KdcAccessibility.bads.contains(s);
        }
        
        private static synchronized void reset() {
            if (KdcComm.DEBUG) {
                System.out.println(">>> KdcAccessibility: reset");
            }
            KdcComm.KdcAccessibility.bads.clear();
        }
        
        private static synchronized List<String> list(String str) {
            StringTokenizer stringTokenizer = new StringTokenizer(str);
            ArrayList<String> list = new ArrayList();
            if (KdcComm.badPolicy == KdcComm.BpType.TRY_LAST) {
                ArrayList<String> list2 = new ArrayList();
                while (stringTokenizer.hasMoreTokens()) {
                    String nextToken = stringTokenizer.nextToken();
                    if (KdcComm.KdcAccessibility.bads.contains(nextToken)) {
                        list2.add(nextToken);
                    }
                    else {
                        list.add(nextToken);
                    }
                }
                list.addAll(list2);
                return list;
            }
            while (stringTokenizer.hasMoreTokens()) {
                list.add(stringTokenizer.nextToken());
            }
            return list;
        }
        
        static void access$000() {
            reset();
        }
        
        static List access$100(String s) {
            return list(s);
        }
        
        static void access$200(String s) {
            removeBad(s);
        }
        
        static void access$300(String s) {
            addBad(s);
        }
        
        static boolean access$400(String s) {
            return isBad(s);
        }
        
        static {
            KdcComm.KdcAccessibility.bads = new HashSet();
        }
    }
}
